--This event is too different from the cue games event class so will be a stand alone, non-implemented version

local M = {}

local helper = require('scenario/scenariohelper')
local eventHandler = require('scenario/cargames/eventHandler')
local uiHandler = require('scenario/cargames/uiHandler')

local scenarioTime = 0
local start = false
local reset = true
local severity = 1
local ballPoints = 0
local ballsPotted = 0
local eventArea = 0
local windTime = 0.01
local alpha = 0
local vehicleObjects = {}
local flip = false
local timeUntilNextStorm = 10
local stormPoints = 0
local stationaryTime = 0

local function onRaceStart()
	if not start then
		start = true
		TorqueScript.eval([[
			triangle_1.position = "10000 0 0";
			triangle_2.position = "10000 0 0";
			triangle_3.position = "10000 0 0";
		]])
		be:reloadStaticCollision();
	end
end

local function success()
	start = false
	eventHandler.finishScenario("cuegames", 4, "Theme 4:3| Cue Games: Instability", nil, {scoreBall = ballPoints, scoreStorms = stormPoints})
end

local function resetRandom(vehicle)
	--[[
	Resets the vehicle/ball to a random position on the table
	]]--
	--These are the bounds on the table
	x, y = math.random(-609, -519), math.random(-1342, -1182)
	TorqueScript.eval(vehicle..'.position = "'..x..' '..y..' 34";')
	TorqueScript.eval(vehicle..'.rotation = "1 0 0 0"; ')
end

local function onBeamNGTrigger(data)
	if (data.triggerName == 'pocket1' or data.triggerName == 'pocket2' or data.triggerName == 'pocket3' or data.triggerName == 'pocket4' or data.triggerName == 'pocket5' or data.triggerName == 'pocket6') and data.event == 'enter' then
		if data.subjectName ~= "scenario_player0" then
			ballPoints = ballPoints+250
			ballsPotted = ballsPotted+1
			resetRandom(data.subjectName)
		else
			--Place the player at the spawn point of the scenario
			TorqueScript.eval(data.subjectName..'.position = "-559.627 -1212.56 30.5";')
		end
	elseif string.find(data.triggerName, "outside") then
		TorqueScript.eval(data.subjectName..'.position = "-559.627 -1212.56 30.5";')
	elseif string.find(data.triggerName, "area") then
		local vehicleObject = scenetree.findObject(data.subjectName)
		if windTime < 10 and windTime ~= 0.01 and tonumber(string.match(data.triggerName, "%d+")) == eventArea then
			if tonumber(string.match(data.triggerName, "%d+")) == eventArea then
				local windSpeed
				if data.subjectName == "scenario_player0" then
					windSpeed = 75*severity
				else
					--The balls have left wind than the player since the wind will destroy the balls everwise
					windSpeed = 20 + (5*(severity-1))
				end

				if eventArea <= 3 then
					vehicleObject:queueLuaCommand('obj:setWind(0, '..windSpeed..', 0)')
				end
			else
				vehicleObject:queueLuaCommand('obj:setWind(0, 0, 0)')
			end
		else
			vehicleObject:queueLuaCommand('obj:setWind(0, 0, 0)')
		end
	end
end

local function getInstanceColour()
	--[[
	Retuerns the rgb of the waypoint based on storm severity
	]]--
	if     severity == 1 then return 0,   255, 0
	elseif severity == 2 then return 255, 255, 0
	elseif severity == 3 then return 255, 174, 0
	elseif severity == 4 then return 255, 0,   0
	elseif severity == 5 then return 255, 255, 255
	else                      return 0,   0,   0
	end
end

local function flashArea()
	--[[
	FLashes the waypoint to indicate a storm is brewing
	]]--
	if alpha > 2 or alpha < 0 then
		flip = not flip
	end

	--Flips the alpha so 0 < alpha < 2
	if flip then
		alpha = alpha-0.1*bullettime.get()
	else
		alpha = alpha+0.1*bullettime.get()
	end

	local r, g, b = getInstanceColour()

	TorqueScript.eval('stormMarker.instanceColor = "'..r..' '..g..' '..b..' '..alpha..'";')
end

local function onPreRender(dt)
	if bullettime.get() > 0 then
		uiHandler.updateMedalUI("cuegames", 4, nil, true, nil, {scoreBall = ballPoints+stormPoints})

		if reset then
			TorqueScript.eval([[
				triangle_1.position = "-567.786926 -1213.0249 29.0229187";
				triangle_2.position = "-559.72052 -1213.27441 29.0229187";
				triangle_3.position = "-563.669373 -1206.63586 29.0327263";
				area1.position = "10000 0 0";
				area2.position = "10000 0 0";
				area3.position = "10000 0 0";
			]])
			be:reloadStaticCollision();
			reset = false
		end

		--Every storm lasts 10 seconds
		if windTime > 10 then
			flashArea()
		--Forces an alpha of 2 after flashing has occurred
		elseif alpha ~= 2 then
			alpha = 2
			--Update with new alpha
			flashArea()
		end
	end
end

local function updateWaypoint(show)
	--[[
	Places or removes the waypoint at the area specified
	TODO - Why exactly are there three waypoints???
	]]--
	if eventArea == 1 then
		if show then
			TorqueScript.eval('stormMarker.position = "-523.622681 -1137.88367 44.4875488";')
		else
			TorqueScript.eval('stormMarker.position = "10000 0 0";')
		end
	elseif eventArea == 2 then
		if show then
			TorqueScript.eval('stormMarker.position = "-563.471436 -1137.61584 44.7102432";')
		else
			TorqueScript.eval('stormMarker.position = "10000 0 0";')
		end
	elseif eventArea == 3 then
		if show then
			TorqueScript.eval('stormMarker.position = "-606.130188 -1137.61584 44.7102432";')
		else
			TorqueScript.eval('stormMarker.position = "10000 0 0";')
		end
	end
end

local function getTimeToNextStorm()
	--[[
	Returns the amount of time before the storm will last for
	]]--
	--Values should be between 13 --> 15, where the more balls potted = less storm time
	if 15-math.floor(ballsPotted/12) >= 13 then
		return 15-math.floor(ballsPotted/12)
	else
		return 13
	end
end

local function initiateNextStorm()
	--[[
	Determines where the next storm will spawn and spawns it
	]]--
	windTime = getTimeToNextStorm()
	eventArea = math.random(1, 3)
	helper.flashUiMessage('Category '..severity..' Storm Forming! ', 3)
	updateWaypoint(true)
	--A storm is already spawned so don't spawn another
	timeUntilNextStorm = -1
end

local function selectSeverity()
	--[[
	Selects a severity for the next storm, based on how many balls have been potted
	]]--
	--If more than 25 balls have been potted, the severity will always be the highest
	if ballsPotted >= 25 then
		severity = 5
	else
		local rand = math.random(0, 100)
		if     rand < 25-(ballsPotted*2) then severity = 1
		elseif rand < 50-(ballsPotted*2) then severity = 2
		elseif rand < 75-(ballsPotted*2) then severity = 3
		elseif rand < 90-(ballsPotted*2) then severity = 4
		else                                  severity = 5
		end
	end
end

local function resetWind()
	--[[
	Resets the wind on the player and the balls
	]]--
	scenetree.findObject("scenario_player0"):queueLuaCommand('obj:setWind(0, 0, 0)')
	--Reset balls
	for i = 0, 9 do
		scenetree.findObject('red'..i):queueLuaCommand('obj:setWind(0, 0, 0)')
	end
end

local function getFrequency()
	--[[
	Returns the time between storms, based on how many balls have been potted
	0 means spawn another storm after one has finished
	]]--
	if 15-ballsPotted > 0 then
		return 15-ballsPotted
	else
		return 0
	end
end

local function resetVars()
	--[[
	Resets the variables after a storm has finished
	]]--
	windTime = 0.01
	alpha = 0
	flip = false
	stormPoints = stormPoints + (10*severity)
	updateWaypoint(false)
	selectSeverity()
	timeUntilNextStorm = getFrequency()
	resetWind()
end

local function timeOut(rtt)
	--[[
	Determines whether the player has been stationary for more than 5 seconds
	]]--
	stationaryTime = stationaryTime+rtt
	if stationaryTime >= 5 then
		return true
	else
		return false
	end
end

local function onRaceTick(raceTickTime)
	if start then
		scenarioTime = scenarioTime+raceTickTime

		--Countdown until next storm
		if timeUntilNextStorm > 0 then
			timeUntilNextStorm = timeUntilNextStorm-raceTickTime
		end

		--Countdown the storm time (> 0.01 as this is the reset variable default)
		if windTime > 0.01 then
			windTime = windTime-raceTickTime
		end

		if windTime == 0 then
			resetVars()
		elseif timeUntilNextStorm == 0 then
			initiateNextStorm()
		end

		local scenario_playerData = map.objects[map.objectNames["scenario_player0"]]
		--The maximum damage the player's car can take before counting as failed
		if scenario_playerData.damage >= 225000 then
			success()
		--Minimum speed to count as stationary
		elseif scenario_playerData.vel:length() <= 0.5 then
			if timeOut(raceTickTime) then
				success()
			end
		elseif stationaryTime > 0 then
			stationaryTime = 0
		end
	end

	-- ui_message('Damage: '..math.floor((scenario_playerData.damage/225000)*100)..'%', 1, 'B')
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onBeamNGTrigger = onBeamNGTrigger
M.onPreRender = onPreRender
return M
