--Best = 1434

local M = {}

local setup, alreadyActive, start = false, 0, false
local location = "grey"
--Stars are at opposite ends to each other in the orbit
local rotations = {star1 = 0, star2 = math.pi}
local moons = {{0, 0, 1, 0, 2, 0, 0, 7, 0, 1}, {1, 0, 0, 2, 0, 0, 1, 1, 4, 0}}
local speeds = {planets = {0.006, 0.0059, 0.0055, 0.0049, 0.0047, 0.0042, 0.0041, 0.0036, 0.0031, 0.0028, 0.0016, 0.0059, 0.0058, 0.0052, 0.005, 0.0045, 0.0043, 0.0042, 0.0034, 0.0031, 0.0026, 0.0014}, moons = {0.01, 0.01, 0.0089, 0.01, 0.0087, 0.0067, 0.0059, 0.005, 0.0035, 0.0027, 0.01, 0.01, 0.01, 0.0068, 0.01, 0.01, 0.01, 0.0068, 0.0046, 0.0034}}

local eventHandler = require("scenario/cargames/eventHandler")

local function onScenarioChange(sc)
	if sc.state == "pre-running" then
		--Reset player position
		TorqueScript.eval('scenario_player0.position = "-564.292236 -3070.60962 3692.60986";')
		start = true
	end
end

local function setupRotations()
	--[[
	Creates the rotation table for the planets and moons, and lets the scenario
	know it can continue via the variable setup
	]]--
	for i = 1, 20 do
		rotations["planet"..i] = 0
		rotations["moon"..i] = 0
	end
	setup = true
end

local function onBeamNGTrigger(data)
	--Various portals to teleport each car at manual positions
	if     data.triggerName == "portal_blue1"   then
		TorqueScript.eval('scenario_player0.position = "-642.946 -1311.18 2923.71";')
	elseif data.triggerName == "portal_blue2"   then
		--This variable is used to know which side the smallest star should act it's gravity on
		location = "green"
		TorqueScript.eval('scenario_player0.position = "-642.946 -617.440 2618.91";')
	elseif data.triggerName == "portal_green1"  then
		TorqueScript.eval('scenario_player0.position = "-484.482 -1311.18 2923.71";')
	elseif data.triggerName == "portal_green2"  then
		--This variable is used to know which side the smallest star should act it's gravity on
		location = "blue"
		TorqueScript.eval('scenario_player0.position = "-484.482 -617.440 2618.91";')
	elseif data.triggerName:find("portal_grey") then
		TorqueScript.eval('scenario_player0.position = "-564.741 109.8970 2297.53";')
	end
end

local function getDistance(position, maxDistance, starRadius)
	--[[
	Determines the distance between the player and the gravity well of the star and
	returns whether the player has gone inside its gravity well
	]]--
	local vehicleObject = scenetree.findObject("scenario_player0")
	local distance = math.floor(math.sqrt(math.pow(vehicleObject:getPosition().x-position.x, 2) + math.pow(vehicleObject:getPosition().y-position.y, 2) + math.pow(vehicleObject:getPosition().z-position.z, 2)))
	if distance <= starRadius then
		eventHandler.endScenario({theme = 3, event = "airborne", name = "Theme 3:5| Airborne: Hyperspeed", failed = true, msg = "You fell into the gravity of a star!"})
	else
		return distance <= maxDistance
	end
end

local function checkDistances()
	--[[
	Checks each stars distance from the player and activates the wind if the player
	has gone inside the gravity well
	]]--
	for i = 1, 5 do
		local range, radius = 375, 300

		--Range and radiues of the smallest star is different
		if i == 5 then
			range = 84
			radius = 42
		end

		local activate = getDistance(scenetree.findObject("star"..i):getPosition(), range, radius)
		if activate then
			if i == 5 then
				if location == "blue" then
					scenetree.findObject("scenario_player0"):queueLuaCommand('obj:setWind(-300, 0, 0)')
				else
					scenetree.findObject("scenario_player0"):queueLuaCommand('obj:setWind(300, 0, 0)')
				end
			else
				--The left handside stars are odd, the right even
				if i % 2 == 0 then
					scenetree.findObject("scenario_player0"):queueLuaCommand('obj:setWind(300, 0, 0)')
				else
					scenetree.findObject("scenario_player0"):queueLuaCommand('obj:setWind(-300, 0, 0)')
				end
			end
			alreadyActive = i
		else
			--If the star is active and player is outside, seise wind
			if alreadyActive == i then
				scenetree.findObject("scenario_player0"):queueLuaCommand('obj:setWind(0, 0, 0)')
				alreadyActive = 0
			end
		end
	end
end

local function onPreRender(dt)
	if start and bullettime.get() > 0 then
		checkDistances()
		if not setup then
			setupRotations()
		else
			local moonsPlaced, planetsPlaced = 0, 0
			for i = 1, 2 do
				rotations["star"..i] = rotations["star"..i] + 0.001*dt*100*bullettime.get()

				local starCenterX, starCenterZ, starRadius = -550, 1500, 1100
				local starX, starY, starZ = starCenterX+starRadius*math.sin(rotations["star"..i]), 3000, starCenterZ+starRadius*math.cos(rotations["star"..i])

				TorqueScript.eval([[
					star_obj]]..i..[[.position = "]]..starX..[[ ]]..starY..[[ ]]..starZ..[[";
				]])

				for j = 1, 10 do
					planetsPlaced = planetsPlaced+1

					rotations["planet"..planetsPlaced] = rotations["planet"..planetsPlaced]+speeds["planets"][planetsPlaced]*dt*100*bullettime.get()
					local planetCenterX, planetCenterZ, planetRadius = starX, starZ, 100*j
					local planetX, planetY, planetZ = planetCenterX+planetRadius*math.sin(rotations["planet"..planetsPlaced]), 3000, planetCenterZ+planetRadius*math.cos(rotations["planet"..planetsPlaced])

					TorqueScript.eval([[
						pathPlanet]]..planetsPlaced..[[.position = "]]..starX..[[ ]]..starY..[[ ]]..starZ..[[";
						pathPlanet]]..planetsPlaced..[[.scale = "]]..(j/2)..[[ ]]..(j/2)..[[ ]]..(j/2)..[[";
						planet_obj]]..planetsPlaced..[[.position = "]]..planetX..[[ ]]..planetY..[[ ]]..planetZ..[[";
					]])

					for k = 1, moons[i][j] do
						moonsPlaced = moonsPlaced+1

						rotations["moon"..moonsPlaced] = rotations["moon"..moonsPlaced]+speeds["moons"][moonsPlaced]*3*dt*100*bullettime.get()
						local moonCenterX, moonCenterZ, moonRadius = planetX, planetZ, 25+(25*k)
						local moonX, moonY, moonZ = moonCenterX+moonRadius*math.sin(rotations["moon"..moonsPlaced]), 3000, moonCenterZ+moonRadius*math.cos(rotations["moon"..moonsPlaced])

						TorqueScript.eval([[
							pathMoon]]..moonsPlaced..[[.position = "]]..planetX..[[ ]]..planetY..[[ ]]..planetZ..[[";
							pathMoon]]..moonsPlaced..[[.scale = "]]..(k/2+1/2)..[[ ]]..(k/2+1/2)..[[ ]]..(k/2+1/2)..[[";
							moon_obj]]..moonsPlaced..[[.position = "]]..moonX..[[ ]]..moonY..[[ ]]..moonZ..[[";
						]])
					end
				end
			end
		end
	end
end

M.onScenarioChange = onScenarioChange
M.onBeamNGTrigger = onBeamNGTrigger
M.onPreRender = onPreRender
return M
