local M = {}

local angles = {}

local function rotateCelestial(name, dt)
  --[[
  Rotates a celestial in the event
  ]]--
  local angleData = angles[name]
  angles[name].val = angleData.val + dt*angleData.freq

  local x, y, z = 0, 0, 0
  if angleData.rotAxis == "x" then
    x = angles[name].val
    y = math.rad(angleData.axisTilt)
  else
    x = math.rad(angleData.axisTilt)
    z = angles[name].val
  end

  local q = quatFromEuler(x, y, z):toTorqueQuat()

  TorqueScript.eval(name..'.rotation = "'..q.x..' '..q.y..' '..q.z..' '..q.w..'";')
end

local function updateRotations(dt)
  --[[
  Updates the rotation for each object in an event
  ]]--
  for name, _ in pairs(angles) do
    rotateCelestial(name, dt)
  end
end

local function getCelestialRotationData(name)
  --[[
  Returns the frequnecy and axis tilt for a celestial
  Note: ringed planets do not return axis tilt since this is scenario dependant
  ]]--
  if     name:find("sun") then
    return 0.4, 7.25
  elseif name:find("planetA") then
    return 0.8, 1.25
  elseif name:find("planetB") then
    return 0.1, 1
  elseif name:find("planetC") then
    return 0.8, 17.8
  elseif name:find("planetD") then
    return 0.1, 3
  elseif name:find("planetE") then
    return 0.35, 4
  elseif name:find("planetF") then
    return 1.2, 5.67
  elseif name:find("planetX") then
    return 4
  elseif name:find("saturn") then
    return 1.6
  end
end

local function setupCelestialRotations()
  --[[
  Sets up the initial values for each celestial body in the event
  ]]--
  local objects = {}

  --Find all celestials
  --These are all different naming conventions used (as some levels require more than one)
  local names = {"star", "planet", "star_obj", "planet_obj"}
  local i = 1
  for _, objName in ipairs(names) do
    i = 1
    local name = objName..i
    while scenetree.findObject(name) do
      table.insert(objects, name)
      i = i + 1
      name = objName..i
    end
  end

  for _, name in ipairs(objects) do
    local data = {}
    local freq, axisTilt = getCelestialRotationData(scenetree.findObject(name).shapeName)

    --Ringed planets have their own axis tilt within the scneario (since it matters which way they face)
    if not axisTilt then
      axisTilt  = scenetree.findObject(name).axisTilt or 0
      data.rotAxis = scenetree.findObject(name).rotAxis or "x"
    end

    data.val = 0
    data.freq = freq
    data.axisTilt = axisTilt

    angles[name] = data
  end
end

M.updateRotations = updateRotations
M.setupCelestialRotations = setupCelestialRotations
return M
