local M = {}

local function setupDartStats(pointsLand, pointsFinal)
	--[[
	Sets up dart statistics
	]]--
	local playerObject = scenetree.findObject('scenario_player0')
	if pointsFinal then
		statistics_statistics.initialiseArbitraryStat("Final Position", "Final Position", be:getPlayerVehicle(0), "scenario_player0", 100, 100)
		statistics_statistics.setStatProgress(playerObject:getID(), "Final Position", "scenario_player0", {value = pointsFinal, maxValue = 100})
	end
	if pointsLand then
		statistics_statistics.initialiseArbitraryStat("Landing", "Landing", be:getPlayerVehicle(0), "scenario_player0", 100, 100)
		statistics_statistics.setStatProgress(playerObject:getID(), "Landing", "scenario_player0", {value = pointsLand, maxValue = 100})
  end
end

local function setupCurlingStats(pointsTeam, pointsOpponent)
	--[[
	Sets up curling statistics
	]]--
	if not pointsTeam or not pointsOpponent then return end
	local playerObject = scenetree.findObject('scenario_player0')
	--Player's Team
	for i = 0, #pointsTeam-1 do
		local name = "Team "..i
		if i == 0 then name = "Player" end

		statistics_statistics.initialiseArbitraryStat(name, name, playerObject, "scenario_player0", 100, 100)
		statistics_statistics.setStatProgress(playerObject:getID(), name, "scenario_player0", {value = pointsTeam[i+1], maxValue = 100})
	end

	--Opponents' Team
	for i = 0, #pointsOpponent-1 do
		local name = "Opponent "..i

		statistics_statistics.initialiseArbitraryStat(name, name, playerObject, "scenario_player0", -100, 100)
		statistics_statistics.setStatProgress(playerObject:getID(), name, "scenario_player0", {value = pointsOpponent[i+1], maxValue = 100})
	end
end

local function setupCuegamesStats(pointsBall, pointsTime, lostPoints)
	--[[
	Sets up cue games statistics
	]]--
	local playerObject = scenetree.findObject('scenario_player0')
	statistics_statistics.initialiseArbitraryStat("Balls", "Balls", be:getPlayerVehicle(0), "scenario_player0", pointsBall["maxPts"], pointsBall["maxVal"])
	statistics_statistics.setStatProgress(playerObject:getID(), "Balls", "scenario_player0", {value = pointsBall["pts"], maxValue = pointsBall["maxVal"]})

	if pointsTime then
		statistics_statistics.initialiseArbitraryStat("Time", "Time", be:getPlayerVehicle(0), "scenario_player0", pointsTime["maxPts"], pointsTime["maxVal"])
		statistics_statistics.setStatProgress(playerObject:getID(), "Time", "scenario_player0", {value = pointsTime["pts"], maxValue = pointsTime["maxVal"]})
	end

	if lostPoints then
		statistics_statistics.initialiseArbitraryStat("Destroyed", "Destroyed", be:getPlayerVehicle(0), "scenario_player0", -lostPoints["maxPts"], lostPoints["maxVal"])
		statistics_statistics.setStatProgress(playerObject:getID(), "Destroyed", "scenario_player0", {value = lostPoints["pts"], maxValue = lostPoints["maxVal"]})
	end
end

--Both paramaters should be arrays since involve different bounds of points per theme
local function setupTargetsStats(pointsTarget, pointsTime)
	--[[
	Sets up targets statistics
	]]--
	local playerObject = scenetree.findObject('scenario_player0')
	statistics_statistics.initialiseArbitraryStat("Targets", "Targets", be:getPlayerVehicle(0), "scenario_player0", pointsTarget["maxPts"], pointsTarget["maxVal"])
	statistics_statistics.initialiseArbitraryStat("Time", "Time", be:getPlayerVehicle(0), "scenario_player0", pointsTime["maxPts"], pointsTime["maxVal"])

	statistics_statistics.setStatProgress(playerObject:getID(), "Targets", "scenario_player0", {value = pointsTarget["pts"], maxValue = pointsTarget["maxVal"]})
  statistics_statistics.setStatProgress(playerObject:getID(), "Time", "scenario_player0", {value = pointsTime["pts"], maxValue = pointsTime["maxVal"]})
end

local function setupAirborneStats(scoreDistance, maxDistance, scoreSpeed, maxSpeed)
	--[[
	Sets up airborne statistics
	]]--
	local playerObject = scenetree.findObject('scenario_player0')
	statistics_statistics.initialiseArbitraryStat("Distance", "Distance", be:getPlayerVehicle(0), "scenario_player0", maxDistance, maxDistance)
	statistics_statistics.setStatProgress(playerObject:getID(), "Distance", "scenario_player0", {value = scoreDistance, maxValue = maxDistance})

	if maxSpeed then
		statistics_statistics.initialiseArbitraryStat("Speed", "Speed", playerObject, "scenario_player0", maxSpeed, maxSpeed)
		statistics_statistics.setStatProgress(playerObject:getID(), "Speed", "scenario_player0", {value = scoreSpeed, maxValue = maxSpeed})
	end
end

local function setupMultiplierStats(multiplier, maxVal, maxMulti)
	--[[
	Sets up multiplier statistics
	]]--
	local playerObject = scenetree.findObject('scenario_player0')
	statistics_statistics.initialiseArbitraryStat("Multiplier", "Multiplier", be:getPlayerVehicle(0), "scenario_player0", maxMulti, maxVal)
	statistics_statistics.setStatProgress(playerObject:getID(), "Multiplier", "scenario_player0", {value = multiplier, maxValue = maxVal})
end

M.setupDartStats = setupDartStats
M.setupCurlingStats = setupCurlingStats
M.setupCuegamesStats = setupCuegamesStats
M.setupTargetsStats = setupTargetsStats
M.setupAirborneStats = setupAirborneStats
M.setupMultiplierStats = setupMultiplierStats
return M
