local M = {}

local file  = require('scenario/cargames/fileHandler')
local event = require('scenario/cargames/eventHandler')

local function getNextMedalAndScores(currentScore, theme, eventNum)
	--[[
	Returns the next medal to achieve - if they have diamond, then return their highest score
	]]--
	local medals = event.getMedalScore(theme, eventNum)
	local bestMedal = file.getAttributeFromSave({theme = theme, event = eventNum, value = "medal"})
	local bestScore = file.getAttributeFromSave({theme = theme, event = eventNum, value = "score"})

	local targetMedal, targetScore
	--If player's medal is diamond, the target score is their personal best
	if bestMedal == "diamond"  then
		targetMedal = "diamond"
		targetScore = bestScore
	--Else, find out the next best medal from their current and the score associated with that medal
	else
		targetMedal = file.getMedalName(file.getMedalRanking(bestMedal)+1)
		targetScore = medals[file.getMedalRanking(bestMedal)+1]
	end

	local currentMedal
	for i = 1, 5 do
		if currentScore < medals[i] then
			--The current medal achieved must be the one below that
			currentMedal = file.getMedalName(i-1)
			if file.getMedalRanking(currentMedal) >= file.getMedalRanking(targetMedal) then
				targetMedal = file.getMedalName(i)
				targetScore = medals[i]
			end
			break
		end
	end
	--If no medal can be mapped, they must have a diamond medal
	if currentMedal == nil then
		currentMedal = "diamond"
		--Set the target medal to be diamond to show that this is the best medal to get
		targetMedal = "diamond"

		local diamondMedalScore = medals[5]
		--If the player got a better diamond medal score, show that instead
		if diamondMedalScore < bestScore then
			targetScore = bestScore
		else
			targetScore = diamondMedalScore
		end
	end

	--Determine the relative file locations for each of the medal
	currentMedal = "modules/apps/Scoring/"..currentMedal..".png"
	targetMedal  = "modules/apps/Scoring/"..targetMedal..".png"
	bestMedal    = "modules/apps/Scoring/"..bestMedal..".png"

	return currentMedal, targetMedal, targetScore, bestMedal, bestScore
end

local function updateNitroUI(nitroTotal, nitroCollected, dt, blink)
	--[[
	Updates the nitro UI with the required information
	]]--
	guihooks.trigger("nitroIndicator", {nitro = math.floor(nitroCollected/nitroTotal*100)/100, increment = dt, active = blink})
end

local function updateMedalUI(eventType, theme, powerups, required, score, ...)
	--[[
	Updates the medal UI with the required information
	]]--
	local currentScore = score or 0

	--If the scenario is tracking realtime scores
	if required then
		currentScore = event.getScores(false, {event = eventType, theme = theme}, powerups, ...).scores.total
	end

	local currentMedal, targetMedal, targetScore, bestMedal, bestScore = getNextMedalAndScores(currentScore, theme, file.determineEventFromFile(eventType))
	guihooks.trigger("scoringScore", {current = {score = currentScore, medal = currentMedal}, best = {score = bestScore, medal = bestMedal}, target = {score = targetScore, medal = targetMedal}})
end

local function updateTimerUI(scenarioTime, timerStart, eventType)
	--[[
	Updates the timer UI with the required information
	]]--
	local timeLeft = timerStart - scenarioTime

	guihooks.trigger("timeLeft", {time = timeLeft/timerStart, event = eventType})
end

local function updatePowerups(powerupTriggered, powerups, override)
	--[[
	Updates the powerups UI with the new powerup and also converts so that JS understands
	it better
	]]--
	local powerupsNew
	if powerups then
		--This is a TEMPORARY hack to make this below code work while darts 5:1 has
		--not been fully converted to new class-style format
		if not override then
			powerupsNew = {}
			for k, v in pairs(powerups) do
				table.insert(powerupsNew, {type = v:getType(), id = v:getId()})
			end
		else
			powerupsNew = powerups
		end
	end

	guihooks.trigger("powerupIndicator", {initialise = powerupsNew, powerupTriggered = powerupTriggered})
end

M.updateNitroUI = updateNitroUI
M.updateMedalUI = updateMedalUI
M.updateTimerUI = updateTimerUI
M.updatePowerups = updatePowerups
return M
