angular.module('beamng.apps')
.directive('eventEndScreen', ['bngApi', 'gamepadNav', function (bngApi, gamepadNav) {
  return {
    templateUrl: 'modules/apps/EventEndScreen/templateBMNG.html',
    replace: true,
    link: function (scope, element, attrs) {
      'use strict';

      scope.$on("carGamesEndScreen", function (event, data) {
        /*
        *Creates a row at the end of the data table
        *@param id - the id of the newly created row
        */
        function createRowAtEnd(id) {
          var newRow = document.getElementById("pointsToAdd").insertRow(-1);
          newRow.id = "row_" + id;

          for (var i = 0; i < 2; i++) {
            var cell  = newRow.insertCell(0);
            cell.innerHTML = "&nbsp";
          }
        }

        /*
        *Updates the a row's cells in the results table
        *@param id     - the id of the newly created row
        *@param reason - the reason for the points awarded (left cell)
        *@param amount - the amount of points awarded (right cell)
        *@param changeReason (optional) - why the change occured (only used for total)
        *@param preColoured (optional) - when true, used to stop this function from overwritting colours
        */
        function updateRow(id, reason, amount, changeReason, preColoured) {
          var row = document.getElementById("row_" + id);
          if (!row) {
            return;
          }

          var cells = row.cells;

          var left = cells[0];
          var right = cells[1];

          if (reason === "Total") {
            var animation = "";
            if (changeReason === "negative") {
              animation = "glowRed 0.5s";
            } else if (changeReason === "positive") {
              animation = "glowGreen 0.5s";
            } else if (changeReason === "neutral") {
              animation = "glowBlue 0.5s";
            } else if (changeReason === "multiplier") {
              animation = "addMultiplier 1s";
            } else {
              console.log("Total updated with unknown reasoning...");
            }

            // Add bold tags + animation
            left.innerHTML = "<span style='animation:" + animation + "; font-size:32px; color:white;'><b>" + reason + "</b></span>";
            right.innerHTML = "<span style='animation:" + animation + "; font-size:32px; color:white;'><b>" + amount + "</b></span>";
          } else {
            if (preColoured) {
              left.innerHTML = "<b>" + reason;
              right.innerHTML = "<b>" + amount;
            } else {
              if (amount < 0 || reason === "Failure" || amount === "Failure") {
                left.innerHTML = "<span style='color:red;'><b>" + reason + "</span>";
                right.innerHTML = "<span style='color:red;'><b>" + amount + "</span>";
              } else if (amount > 0 || reason === "Success" || amount === "Success") {
                if (reason === "Multiplier") {
                  left.innerHTML = "<span style='color:orange;'><b>" + reason + "</b></span>";
                  right.innerHTML = "<span style='color:orange;'><b>" + amount + "</b></span>";
                } else {
                  left.innerHTML = "<span style='color:#00FF00;'><b>" + reason + "</b></span>";
                  right.innerHTML = "<span style='color:#00FF00;'><b>" + amount + "</b></span>";
                }
              } else {
                left.innerHTML = "<span style='color:#005FFF;'><b>" + reason + "</b></span>";
                right.innerHTML = "<span style='color:#005FFF;'><b>" + amount + "</b></span>";
              }
            }
          }
        }

        /*
        Adds the multipliers to the table and applies them to the score
        *@param multipliers - an array of multipliers gotten
        *@param currentTotal - where the total left off from accumulating
        *@param rowId - the id of the powerup row in the table
        */
        function addMultipliers(multipliers, currentTotal, rowId) {
          var cont = false;
          // Waits for a half-second before calling the next function
          var wait = function() {
            if (cont) {
              showRanking(currentTotal);
            } else {
              cont = true;
              setTimeout(wait, 500);
            }
          }

          if (multipliers) {
            var row = document.getElementById("row_" + rowId);
            if (!row) {
              return;
            }

            var cells = row.cells;

            cells[0].innerHTML = "<span style='color:orange'><b>Multiplier</b></span>";
            cells[1].innerHTML = "";

            var index = 0;
            var timer = setInterval(function() {
              var current = multipliers[index];
              cells[1].innerHTML += "<img src='modules/apps/EventEndScreen/multipliers/x" + current + ".png' style='height:28px; width:32px;'>";

              currentTotal *= current;
              // Total row is 2 rows down from the multiplier row
              updateRow(rowId+2, "Total", currentTotal, "multiplier");

              index++;

              if (index >= multipliers.length) {
                clearInterval(timer);
                wait();
              }
            }, 500);
          } else {
            wait();
          }
        }

        /*
        *Fills the points table with those awarded
        *@param data - the array of point, reason pairs to add to the table
        */
        function fillPoints(data) {
          // A function to wait 1/2 second before continuing
          var cont = false;
          var wait = function(){
            if (cont) {
              addMultipliers(multipliers, total, rowId);
            } else {
              cont = true;
              setTimeout(wait, 500);
            }
          }

          // Used for callback function's arguments
          var multipliers = null;
          var rowId = null;

          if (success) {
            // Create rows beforehand
            for (var i = 0; i < data.length; i++) {
              if (i === data.length-1) {
                // Create blank row between breakdown and total
                createRowAtEnd(i);
                createRowAtEnd(i+1);
                // Last index anyway so may as well break earlier
                break;
              } else {
                createRowAtEnd(i);
              }
            }

            // if (data.length > 7) {
            //   var element = document.getElementById("scoreCounter");
            //   var base = 480;
            //   // Move the score counter down by 40px per row above 7
            //   var increase = 40 * (data.length-6)
            //   element.style.top = (base + increase) + "px";
            // }

            var index = 0;
            var total = 0;

            var timer = setInterval(function() {
              var pts = data[index];
              var reason = pts.reason;
              var amount = pts.amount;

              if (reason === "Multiplier") {
                multipliers = amount;
                rowId = index;
              } else {
                // Update the row's cells
                updateRow(index, reason, amount);

                var reasoning = "";

                // Determine reason for total update changes
                if (amount > 0) {
                  reasoning = "positive";
                } else if (amount < 0) {
                  reasoning = "negative";
                } else if (amount === 0) {
                  reasoning = "neutral";
                }

                total += amount;

                updateRow(data.length, "Total", total, reasoning);
              }

              index++;

              // Ignore last element as it will be the total row, which is not relevant
              if (index === data.length-1) {
                clearInterval(timer);

                //  Make sure that total does not finish below 0
                if (total < 0) {
                  total = 0;
                  updateRow(data.length, "Total", 0, "multiplier");
                }

                wait();
              }
            }, 600);
          } else {
            // Create a row to explain why user failed
            createRowAtEnd(0);
            updateRow(0, "Failure", reason);
            wait();
          }
        }

        /*
        *Shows the player's progress through the championship by replacing the
        *breakdown with the championship progress as well as adjusts the retry
        *button to allow/disallow and show how many retries are available
        */
        function showChampionshipProgress() {
          // Remove breakdown information
          document.getElementById("pointsToAdd").innerHTML = "";

          var index = 0;
          var timer = setInterval(function() {
            createRowAtEnd(index);

            // Add in the information
            var data = championshipData[index];

            if (typeof(data.value) === "object") {
              var value = "";
              for (var i = 0; i < data.value.length; i++) {
                var points = data.value[i];
                value += "<span style='color:" + medalToColour(null, points) + ";'>" + points + "</span>";

                if (i < data.value.length) {
                  value += " ";
                }
              }

              updateRow(index, data.title, value, null, true);
            } else {
              updateRow(index, data.title, data.value);
            }


            index++;

            if (index === 2) {
              clearInterval(timer);

              var finish = championshipData[index];
              if (finish) {
                createRowAtEnd(3);
                createRowAtEnd(4);
                updateRow(4, finish.status, finish.msg);
              }
            }
          }, 600);
        }

        /*
        *Shows the personal best banner
        */
        function showPB() {
          document.getElementById("pbBox").style.display = "block";
        }

        // Length of the medal bar element
        const MEDAL_UI_LENGTH = 820;
        // The width of the image / 2
        const MEDAL_OFFSET = 53/2;

        /*
        *Shows the bottom buttons
        */
        function showButtons() {
          document.getElementById("buttons").style.display = "block";

          if (inChapionshipMode) {
            // Remove retry button in championship mode
            document.getElementById("retry").style.display = "none";
          } else {
            document.getElementById("retry").addEventListener("click", restartScenario, false);
          }

          let filePath = "levels/smallgridspace/scenarios/cargames/roam/cargameshub.json";
          document.getElementById("hubWorld").addEventListener("click", function () {loadScenario(filePath);});

          if (data.scenarioData.nextScenario) {
            document.getElementById("nextScenario").addEventListener("click", function() {loadScenario(data.scenarioData.nextScenario);})
          } else {
            document.getElementById("nextScenario").style.display = "none";
          }
        }
        /*
        *Shows the medal achieved
        *@param medalAchieved - the medal achieved
        *@param betterMedal - whether this is a better medal than previous
        *@param betterScore - whether this is a better score than previous
        */
        function showMedal(medalAchieved, betterMedal, betterScore) {
          if (medalAchieved === "") {
            medalAchieved = "fail";
          }

          var medalElement = document.getElementById("medal");
          medalElement.style.display = "block";
          medalElement.src = "modules/apps/EventEndScreen/medals/" + medalAchieved + ".png";
          medalElement.style.animation = "glow 2s infinite forwards, fadeInScale 1.5s";

          if (betterMedal) {
            medalElement.addEventListener("animationend", function() {
              var element = document.getElementById("effect");
              element.style.display = "block";
              element.style.animation = "shimmerBackground 2s linear infinite";

              // var colour = medalToColour(medalAchieved);
              // effect.style.backgroundImage = "linear-gradient(to right, " + colour + "00 0%," + colour + "00 50%," + colour + "21 77%," + colour + "F3 92%," + colour + "00 100%)";
            });
          }

          if (betterScore) {
            showPB();
          }

          if (inChapionshipMode) {
            showChampionshipProgress();
          }

          showButtons();
        }

        /*
        *Shows the player's score compared to the medals'
        *@param total - the total amount of points accumulated
        */
        function showRanking(total) {
          //The total time it should take to animate the timer
          const REAL_TIME = 4;

          var incrementMultiplier = 0;
          var t = 0;
          var currentPos = 0;
          var pendEnd = false;

          var medalAchieved = "";
          var betterScore = false

          // The position this score will go to
          var totalPos = POINTS_FACTOR * total;

          if (success) {
            var timer = setInterval(function() {
              //Since this function updates every 10ms
              t += 1/100;

              //f(t) = (t/K - 1)³ + 1
              incrementMultiplier = Math.pow((t/REAL_TIME - 1), 3) + 1;

              //Account for x³ approaching 1 unlikely to ever be 1 exactly
              if (Math.abs(incrementMultiplier-1) <= Math.pow(10, -6)) {
                incrementMultiplier = 1;
                pendEnd = true;
              }

              currentPos = Math.min(totalPos, MEDAL_UI_LENGTH) * incrementMultiplier;

              for (let [key, data] of Object.entries(medalInfo)) {
                if (currentPos >= data.pos && key !== "best") {
                  document.getElementById(key + "_line").style.border = "2px solid black";
                  // Change bar colour
                  document.getElementById("playerScore").style.backgroundColor = medalToColour(key);

                  if (!data.gotten) {
                    // Animate collection
                    document.getElementById("medal_" + key).style.animation = "floatDownAndOut 2s forwards";
                  }

                  medalAchieved = key;

                  // Remove the key as this medal has been achieved
                  delete medalInfo[key];
                  // Better than best means greater than, unlike medals which is inclusive of their target
                } else if (currentPos > data.pos) {
                  betterScore = true;
                  // Remove the key as this medal has been achieved
                  delete medalInfo[key];
                }
              }

              document.getElementById("playerScore").style.width = currentPos + "px";

              if (pendEnd) {
                clearInterval(timer);
                showMedal(medalAchieved, betterMedal, betterScore);
              }
            }, 10);
          } else {
            showMedal(medalAchieved, betterMedal, betterScore);
          }
        }

        /*
        *toggles the UI from view
        */
        function toggleUI() {
          var info = document.getElementById("info");
          var button = document.getElementById("toggle");

          if (info.style.visibility === "visible") {
            info.style.visibility = "hidden";

            button.innerHTML = "&nbsp;show&nbsp;";
            button.className = "show";
          } else {
            info.style.visibility = "visible";

            button.innerHTML = "&nbsp;hide&nbsp;";
            button.className = "hide";
          }
        }

        /*
        *Returns the specific colour of a medal
        *@param medal - the name of the medal
        *@param medalValue - the value of the medal
        *Note: only one parameter is necessary
        */
        function medalToColour(medal, medalValue) {
          medal = medal ? medal.toLowerCase() : "";
          medalValue = medalValue ? medalValue : 0;

          if        (medal === "bronze"   || medalValue === 10) {
            return "#CD7F32";
          } else if (medal === "silver"   || medalValue === 12) {
            return "#C0C0C0";
          } else if (medal === "gold"     || medalValue === 15) {
            return "#FFD700";
          } else if (medal === "platinum" || medalValue === 20) {
            return "#E5E4E2";
          } else if (medal === "diamond"  || medalValue === 25) {
            return "#B9F2FF";
          } else if (medal === "best") {
            return "#00ff00";
          } else {
            return "#FF0000";
          }
        }

        /*
        *Returns the colour of a theme
        */
        function themeToColour(theme) {
          switch(theme) {
            case 1:
            return "#00FF00";
            break;
            case 2:
            return "#0067FF";
            break;
            case 3:
            return "#6C6C6C";
            break;
            case 4:
            return "#2ADDFF";
            break;
            case 5:
            return "#E1A241";
            break;
          }
        }

        /*
        *Restarts the current scenario
        */
        function restartScenario() {
          bngApi.engineLua("scenario_scenarios.restartScenario()");
        }

        /*
        * Loads a given scenario
        */
        function loadScenario(filePath) {
          filePath = bngApi.serializeToLua(filePath);
          var cmd = "scenario_scenariosLoader.start(scenario_scenariosLoader.loadScenario(" + filePath + ", nil, " + filePath + "))";
          bngApi.engineLua(cmd);
        }

        /*
        *Places the medals on the UI
        *@param medalScores - an array of the scores for each medal
        *return medalScores - the medalScores array with a new attribute for each medal: pos
        */
        function setupRanking(medalScores) {
          for (let [key, data] of Object.entries(medalScores)) {
            // Center the medal
            var pos = POINTS_FACTOR * data.score - MEDAL_OFFSET;

            var imgHTML = "";
            if (!data.gotten && key !== "best") {
              imgHTML = "<div id='" + key + "_wrapper'><img id='medal_" + key + "' src='modules/apps/EventEndScreen/medals/" + key + ".png' style='width:" + MEDAL_OFFSET*2 + "px; height:50px; opacity:1; position:absolute; top:-100%; left:" + pos + "px;'><span id='" + key + "_line' style='border:2px solid " + medalToColour(key) + "; height:80px; position:absolute; top:-10px; left:" + (pos + MEDAL_OFFSET - 2) + "px;'></span></img></div>";
            } else {
              imgHTML = "<div id='" + key + "_wrapper'><span id='" + key + "_line' style='border:2px solid " + medalToColour(key) + "; height:80px; position:absolute; top:-10px; left:" + (pos + MEDAL_OFFSET - 2) + "px;'></span></div>";
            }

            document.getElementById("scoreCounter").innerHTML += imgHTML;

            // Change to array and add pos to it
            medalScores[key].pos = pos + MEDAL_OFFSET;
          }

          return medalScores;
        }

        function fillData() {
          fillPoints(results);

          document.getElementById("info").removeEventListener("animationend", fillData);
          document.getElementById("toggle").addEventListener("click", toggleUI, false);
        }

        function start() {
          // Change title + colour scheme acordingly
          var title = document.getElementById("scenarioTitle");
          title.style.color = themeColour;
          title.innerHTML = "<b>" + scenarioTitle.toUpperCase() + "</b>";

          document.getElementById("resultsHeading").style.backgroundColor = themeColour;

          if (success) {
            document.getElementById("scoreCounter").style.border = "2px solid " + themeColour;
          } else {
            //Remove outline as this will be empty
            document.getElementById("scoreCounter").style.border = "";
          }

          //Animate finish div
          var finish = document.getElementById("finish");
          finish.style.display = "block";
          finish.style.animation = "slideDownOut 3s linear forwards";

          //Animate results div after the finish div
          var info = document.getElementById("info");
          info.style.display = "block";
          info.style.animation = "slideDownPause 1.75s linear 2.5s forwards";
          //Add event listener to call the function for showing the breakdown of results
          info.addEventListener("animationend", fillData, false);

          //Animate the button to do with the results div
          var button = document.getElementById("toggle");
          button.style.display = "block";
          button.style.animation = "slideDownPauseButton 1.75s linear 2.5s forwards";
        }

        //MAIN

        var success = data.scenarioData.status.success;
        var reason = data.scenarioData.status.reason;

        var inChapionshipMode = data.scenarioData.isInChapionshipMode;
        var championshipData = data.scenarioData.championshipData;

        var theme = data.scenarioData.theme;
        var scenarioTitle = data.scenarioData.name;
        var themeColour = themeToColour(theme);

        // Placeholder variables that are only specificed by Lua if successful
        var results, unlocks, total, betterMedal, medalInfo
        var medalScores = data.medalData;

        var POINTS_FACTOR = success
        // The ratio of score to pixels - *1.05 for gap past diamond/best
        ? MEDAL_UI_LENGTH /(Math.max(medalScores.diamond.score, medalScores.best.score)*1.05)
        : 0;

        var finish = document.getElementById("finish");

        if (!success) {
          finish.style.webkitTextStrokeColor = "lightred";
          finish.style.color = "red";
          finish.innerHTML = "FAIL";
          document.getElementById("medal").style.display = "none";
        } else {
          finish.innerHTML = "FINISHED";

          results = data.resultsData;
          unlocks = data.scenarioData.unlocks;

          // Last index will be total
          total = results[results.length-1].amount;
          betterMedal = medalScores.best.newMedal;

          medalInfo = setupRanking(medalScores);
        }

        start();
      });
    }
  };
}]);
